<?php namespace EwTools\Admin;

use EwTools\Support\Str;

/**
 * Message
 *
 * @link    http://infomaker.se
 * @package EwTools\Admin
 * @since   EwTools\Admin\Message 0.1
 */
class Message {
    
    const ERROR = 'error';
    const WARNING = 'warning';
    const SUCCESS = 'success';
    const INFO = 'info';
    
    /**
     * Available types of the message
     *
     * @var array
     */
    private static $types = [
        self::ERROR,
        self::WARNING,
        self::INFO,
        self::SUCCESS
    ];
    
    /**
     * A dismissible message will automatically trigger a closing icon to be added to your message via JavaScript
     *
     * @var bool
     */
    private $dismissible;
    
    /**
     * Message text
     *
     * @var string
     */
    private $message;
    
    /**
     * Message type
     *
     * @var string
     */
    private $type;
    
    /**
     * Message constructor.
     *
     * @since 0.1
     *
     * @param string $type One of valid types
     * @param string $message
     * @param bool   $dismissible
     */
    public function __construct( $type, $message, $dismissible = false ) {
        
        // If type isn't valid
        if( ! in_array( strtolower($type), static::$types, true ) ) {
            $this->invalidTypeError( $type, $message );
        } else {
            $this->message     = $message;
            $this->type        = strtolower($type);
            $this->dismissible = $dismissible;
        }
    }
    
    /**
     * Static function for creating messages and add it to the message bag
     *
     * @param string $type
     * @param string $message
     * @param bool   $dismissible
     *
     * @since 0.1
     * @return static
     */
    public static function create( $type, $message='', $dismissible = false ) {
        $instance = new static($type, $message, $dismissible);
        MessageBag::add($instance);
        return $instance;
    }
    
    /**
     * Generate html out of all messages
     *
     * @since 0.1
     * @return string
     */
    public static function generateAll() {
        return MessageBag::messagesToHtml();
    }
    
    /**
     * Generate HTML out of the message
     *
     * @since 0.1
     * @return string
     */
    public function toHtml() {
        return sprintf( '<div class="%s"><p>%s</p></div>', $this->getMessageClasses(), $this->getMessage() );
    }
    
    /**
     * Retrieve the message
     *
     * @since 0.1
     * @return string
     */
    public function getMessage() {
        return $this->message;
    }
    
    /**
     * Generate a classes for
     *
     * @since 0.1
     * @return string
     */
    private function getMessageClasses() {
        return implode( ' ', array_filter( [
            'notice',
            Str::append( $this->type, 'notice', '-' ),
            $this->dismissible ? 'is-dismissible' : ''
        ] ) );
    }
    
    /**
     * Create a fallback error message do explain
     *
     * @param string $type
     * @param string $message
     *
     * @since 0.1
     * @return void
     */
    private function invalidTypeError( $type, $message ) {
        $this->message     = sprintf( 'Failed to create message: "%1$s" with type: "%2$s"', $message, $type );
        $this->type        = self::ERROR;
        $this->dismissible = false;
    }
}