<?php namespace EwTools\Storage;

use ArrayAccess;
use EwTools\Interfaces\Support\Arrayable;
use EwTools\Support\Collection;

/**
 * CollectionDB
 *
 * @link    http://infomaker.se
 * @package EwTools\Storage
 * @since   EwTools\Storage\CollectionDB 0.1
 */
class CollectionDB {
    
    /**
     * Contains the option name where the collection is stored
     *
     * @var string
     */
    protected $option_name;
    
    /**
     * Contains whether the collection is global
     *
     * @var bool
     */
    protected $global = false;
    
    /**
     * Contains the data collection
     *
     * @var Collection
     */
    private $collection;
    
    /**
     * CollectionDB constructor.
     *
     * @since 0.1
     *
     * @param string $option_name
     * @param bool   $global
     */
    public function __construct( $option_name, $global = false ) {
        $this->option_name = $option_name;
        $this->global      = $global;
        $this->setCollection( $this->getCollection() );
    }
    
    /**
     * Retrieve data from ObjectDB depending on global state and convert it to a collection before sending it back
     *
     * @since 0.1
     * @return Collection
     */
    private function getCollection() {
        return $this->global ? ObjectDB::getGlobal( null, $this->option_name, [] ) : ObjectDB::get( $this->option_name, [] );
    }
    
    /**
     * A way to set the collection
     *
     * @param array $collection
     *
     * @since 0.1
     * @return CollectionDB
     */
    public function setCollection( $collection=[] ) {
        $this->collection = collect( $collection );
        
        return $this;
    }
    
    /**
     * Retrieve the hole collection of data
     *
     * @since 0.1
     * @return Collection
     */
    public function all() {
        return $this->collection;
    }
    
    /**
     * Add an item to the collection
     *
     * @param mixed $item
     *
     * @since 0.1
     * @return CollectionDB
     */
    public function add( $item ) {
        $this->collection->push( $item );
        
        return $this;
    }
    
    /**
     * Merge items into the collection
     *
     * @param $items
     *
     * @since 0.1
     * @return Collection
     */
    public function merge( $items ) {
        $this->setCollection( $this->collection->merge( $items ) );
        
        return $this->all();
    }
    
    /**
     * Remove an item from the collection
     *
     * @param $key
     *
     * @since 0.1
     * @return CollectionDB
     */
    public function remove( $key ) {
        $this->collection->forget( $key );
        
        return $this;
    }
    
    /**
     * Reset collection
     *
     * @since 0.1
     * @return CollectionDB
     */
    public function reset() {
        return $this->setCollection();
    }
    
    /**
     * Update an existing item in the collection
     *
     * @param $key
     * @param $item
     *
     * @since 0.1
     * @return CollectionDB
     */
    public function update( $key, $item ) {
        $this->collection->put( $key, $item );
        
        return $this;
    }
    
    /**
     * Save collection state to database
     *
     * @since 0.1
     * @return bool
     */
    public function save() {
        return $this->global ? ObjectDB::updateGlobal( null, $this->option_name, $this->collection ) : ObjectDB::update( $this->option_name, $this->collection );
    }
    
    /**
     * Create a collection table with optional data
     *
     * @param string $option_name
     * @param bool   $global
     * @param array  $items
     *
     * @since 0.1
     * @return CollectionDB
     */
    public static function create( $option_name, $global = false, array $items = [] ) {
        $collection = new static( $option_name, $global );
        $collection->setCollection( $items )->save();
        
        return $collection;
    }
    
    /**
     * Drop the collection table
     *
     * @param string $option_name
     * @param bool   $global
     *
     * @since 0.1
     * @return bool
     */
    public static function drop( $option_name, $global = false ) {
        return $global ? ObjectDB::removeGlobal( null, $option_name ) : ObjectDB::remove( $option_name );
    }
    
    /**
     * Static function to connect to the options table and retrieve all data as collection
     *
     * @param string $option_name
     *
     * @since 0.1
     * @return Collection
     */
    public static function table( $option_name ) {
        return ( new static( $option_name ) )->all();
    }
    
    /**
     * Static function to connect to the global options table and retrieve all data as collection
     *
     * @param string $option_name
     *
     * @since 0.1
     * @return Collection
     */
    public static function GlobalTable( $option_name ) {
        return ( new static( $option_name, true ) )->all();
    }
}
